!-------------------------------------------------------------------------------------------------------------
! File: myweldinglibrary.f90
! Written by: Sebastien Rouquette (SR)
! Modified by: SR, the 23rd of february 2013
! date: 19th february 2013
!------------------------------------------------------------------------------------------------------------

!------------------------------------------------------------------------------------------------------------
! Aim: generate a gaussian heat flux on a 1D surface in order ! to simulate the heat flux ! coming from an gt (gas tungsten) ! electric arc plasma
!------------------------------------------------------------------------------------------------------------
FUNCTION weldingspot(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   INTEGER :: n, i, j
   REAL :: Is, Us, eta, Rb, Spotduration, xc
   REAL(KIND=dp) :: t, f, x, y

   Is=110 !welding intensity
   Us=10. !welding current
   eta=0.68 ! welding process efficiency
   Rb=0.01 !gaussian radius
   xc = 0.0
   Spotduration=14.1 !duration of electric arc in seconds

   x = Model % Nodes % x(n)
   y = Model % Nodes % y(n)

! welding heat flux described with a gaussian distribution
   IF (t <= Spotduration) THEN
     f = 10*eta*((Is*Us)/(2*3.14*Rb**2))*exp(-0.5*((x-xc)/Rb)**2)
   ELSE
     f = 0   
   END IF

END FUNCTION weldingspot

!------------------------------------------------------------------------------------------------------------
! Aim: temperature dependent surface tension coefficient
! formulation used: Sahoo law
!------------------------------------------------------------------------------------------------------------
FUNCTION surfacetension(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   REAL :: SFTm, Asft, Tliquidus, Rgas, Gs, k1, Asulfur, dH0
   REAL(KIND=dp) :: t, f

SFTm=1.943 ! [N/m] surface tension for pure metal
Asft=0.00043 ! [N/(m.K)]
Tliquidus=1723. ! [K] 
Rgas=8314.3 ! [J/(kg.mol.K)] Gas constant
Gs=1.3e-8 ! [J/(kg.mol.m^2)] surface excess at saturation
k1=0.00318 ! ???
Asulfur=0.0046 ! wt% sulfur activity
dH0=-1.88e8 ! [J/(kg.mol)] standard heat of adsorption

! Sahoo law
   IF (t > Tliquidus) THEN
     f = SFTm-Asft*(t-Tliquidus)-Rgas*t*Gs*log(1+k1*Asulfur*exp(-dH0/(Rgas*t)))
   ELSE
     t = Tliquidus
     f = SFTm-Asft*(t-Tliquidus)-Rgas*t*Gs*log(1+k1*Asulfur*exp(-dH0/(Rgas*t)))
   END IF

END FUNCTION surfacetension

!--------------------------------------------------
!--------------------------------------------------
! Aim: give molten metal viscosity
!--------------------------------------------------
!--------------------------------------------------
FUNCTION myviscosity(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   REAL(KIND=dp) :: t, f, Tsol, Tliq, VISCSOL
   REAL(KIND=dp) :: T3000, VISCLIQ1, VISCLIQ2
   
! ***********input temperature muste be in Kelvin***************
   Tsol = 1653. ! Solidus temperature for SS304
   Tliq = 1723. ! liquidus temperature for SS304
   T4000 = 4000.
   VISCSOL = 1e4 ! Fake viscosity for no molten metal
   VISCLIQ1 = 0.0067 ! real molten metal viscosity at 1723 K
   VISCLIQ2 = 0.05 ! fake metal viscosity at 3000 K, should be 0.0018

   IF (t <= Tsol) THEN
     f = VISCSOL
   ELSE IF (t >= Tliq) THEN
   f = ((VISCLIQ2-VISCLIQ1)/(T4000-Tliq))*(t-Tliq)+VISCLIQ1
        ELSE
   f = ((VISCLIQ1-VISCSOL)/(Tliq-Tsol))*(t-Tsol)+VISCSOL  
   END IF

END FUNCTION myviscosity

!-----------------------------------------------------
! Aim: compute arc drag force due to the plasma shear flow
!-----------------------------------------------------
FUNCTION arcdrag(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   INTEGER :: n
   REAL :: Parc, Rarc
   REAL(KIND=dp) :: x, t, f, Aux

   Parc=90. ! in Pascal, very low value for low welding intensity

   Rarc=0.0025 !spatial distribution

   ! x = Model % Nodes % x(n)
   !Aux = x/Rarc
   Aux = abs(t)/Rarc
   IF ( t >= 0.) THEN
      f = sqrt(Aux)*Parc*EXP(-Aux**2)
   ELSE
      f = - sqrt(Aux)*Parc*EXP(-Aux**2)
   END IF

END FUNCTION arcdrag

!-----------------------------------------------------
! Aim: compute arc pressure due to the plasma pressure
!-----------------------------------------------------
FUNCTION arcpress(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   INTEGER :: n
   REAL :: Parc, Rarc, Iw, musteel, Spotduration
   REAL(KIND=dp) :: t, f, x

! Parc expression from Traidia Thesis
Iw = 110.
musteel = 3.1e-6 ! austenitic stainless steel magnetic permeability
Parc=(0.5*musteel*Iw**2)/(4*3.14**2) ! in Pascal, very low value for low welding intensity

Rarc=0.0017 !spatial distribution

Spotduration=1.41 !duration of electric arc in seconds

x = Model % Nodes % x(n)

IF (t <= Spotduration) THEN
     f = -(Parc/Rarc**2)*EXP(-0.5*(x/Rarc)**2)
   ELSE
     f = 0   
   END IF

END FUNCTION arcpress

!---------------------------------------------------------
!---------------------------------------------------------
! Aim: apply darcy condition (Carman-Kozeny approximation
! derived from Darcy's law)
!---------------------------------------------------------
!---------------------------------------------------------
FUNCTION Darcy(Model, n, t) RESULT(f)
   USE DefUtils
   TYPE(Model_t) :: Model
   TYPE(Variable_t) , POINTER :: Var
   REAL(KIND=dp) :: t, f, Tsol, Tliq, Taux, Cx, baux, fliq
   REAL(KIND=dp) :: term1, term2, term3, AvVisco

! ***********input temperature muste be in Kelvin***************

   Tsol = 1653. ! Solidus temperature for SS304
   Tliq = 1723. ! liquidus temperature in Kelvin for SS304
   
Var => VariableGet( Model % Variables, 'Temperature')
! print *,'current temperature is:', Var % Values(1)

Taux = Var % Values(1) ! current temperature (kelvin! check it)

Cx = 1e10 ! constant depending on morphology and size of dendrites
baux = 0.001 ! constant to avoid division by zero

AvVisco = 0.0065 ! average viscosity of molten metal

IF (t <= Tsol) THEN
     fliq = 0
     f = 0
   ELSE IF (t >= Tliq) THEN
          fliq = 1
          f = 0
        ELSE
          fliq = (t-Tsol)/(Tliq-Tsol)
          term1 = (1-fliq)*(1-fliq)
          term2 = fliq*fliq*fliq
          term3 = term1/(term2+baux)
          f = -AvVisco*Cx*term3*t
   END IF

!print *,'Taux, Cx & t', Taux, Cx, t
!print *,'term1, term2 & f', term1, term2, f

END FUNCTION Darcy
